'''
This code is part of NightmareTyrant; Copyright (C) 2013 Piotr 'ZasVid' Sikora; see NightmareTyrant.py for full notice.
'''

from guts.game_pieces import Terrain, Feature, Item
from guts.game_agents import Agent, Player, item_action_types
from guts.actions import new_action
from debug import debug
from ai import *
from guts.task_resolution import traits, trait_mods

def new_player(name = "you"):
    player = Player(shape = 'player', color = 'player',
                    flags = ['blocking'],
                    traits = {'sight': 7, 'health' : 25, 'stamina': 25,
                              'attitude': 0, 'luck': 3, 'defense': 3, 'confidence': 3, 'skill': 3,
                              'special equipment' : 10}
                    )
    _add_necessary_traits(player)
    _name_piece(player, 'player', name)
    for item_type in ['sword', 'crossbow', 'firebomb', 'flask of spirits', 'calming smokeleaf']:
        item = new_item(item_type)
        player.receive(item)
        player.equip(item)
    return player

def _name_piece(piece, piece_type, name):
    if name is None:
        piece['name'] = piece_type
    else:
        piece['name'] = name




def _add_necessary_traits(piece):
    for trait in traits:
        if not piece.has(trait):
            piece[trait] = 10 if trait == 'health' else 0
        for trait_mod in trait_mods:
            piece[' '.join([trait_mod,trait])] = 0
    for trait in ['health', 'stamina', 'special equipment']:
        if piece.has(trait) and not piece.has('max ' + trait):
            piece['max ' + trait] = piece[trait]

def new_item(item_type, name = None):
    """ Item factory.
    """
    shape, color, flags, traits = item_types.get(item_type,
                ('default', 'default', [], {}))
                #new_action('feedback', {'feedback msg': 'This doesn\'t do anything.'}
    if shape == 'default':
        debug.log("requested undefined item type: " + str(item_type))
    item = Item(shape = shape, color = color, flags = flags, traits = traits)
    _name_piece(item, item_type, name)
    for action_type in item_action_types:
        if item.has(action_type):
            action_key, action_data = traits[action_type]
            item[action_type] = new_action(action_key, action_data)
            item[action_type].piece = item
    item['type'] = item_type
    return item
    
item_types = {# player stuff
              'sword': ('n/a', 'n/a', [], {'melee': ('melee', {'damage': 4, 'success' : ('permanent','damage'), 'advantage': ('permanent','damage')})}),
              'crossbow' : ('n/a', 'metallic gray', [], {'loaded' : True, 'ranged': ('shoot_or_reload', {'damage' : 4, 'success' : ('permanent','damage'), 'advantage': ('permanent','damage')})}),
              'firebomb' : ('n/a', 'fiery', ['close range', 'limited'], {'deployable': ('blow up', {'damage': 5, 'success' : ('permanent','damage'), 'advantage': ('permanent','damage')})}),
              'flask of spirits': ('n/a', 'angry', ['close range', 'limited'], {
                    'consumable': ('trait transfer', {'from' : 'health', 'to': 'fatigue', 'side effect': 'attitude',
                                                       'msg': "You take a swig from your flask of spirits!" ,'msg color':'angry'})}),
              'calming smokeleaf': ('n/a', 'mellow', ['close range', 'limited'], {
                    'activated': ('trait transfer', {'from' : 'health', 'to': 'fatigue', 'side effect': 'caution',
                                                     'msg': "You smoke a pinch of the calming smokeleaf.",
                                                     'msg color': 'mellow'})}),
              #placeholder
              'n/a': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('tactical', 'damage'), 'advantage': ('tactical', 'damage')}),
                                         'ranged': ('shoot_or_reload', {'success' : ('tactical', 'damage'), 'advantage': ('tactical', 'damage')}),
                                         'loaded' : True
                                         }),
              # undead powers
              'dread bite': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('permanent','fatigue'), 'advantage': ('tactical','imbalance')})}),
              'unearthly wail': ('n/a', 'n/a', ['minion'], 
                                 {'melee': ('melee', {'success' : ('temporary','stress'), 'advantage': ('temporary','disability')})}),
              'rusty blade': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('temporary','fatigue')})}),
              'fist': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('temporary','damage')})}),
              'pebble': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('temporary','damage'), 'advantage': ('tactical','imbalance')})}),
              'bone wand': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('permanent','damage')}),
                                                     'ranged': ('shoot', {'success' : ('temporary','damage'), 'advantage': ('permanent','fatigue')}),
                                                     #'cast' : ('summon', {'what': ['skeleton', 'zombie'], 'where' : 'sarcophagus'}),
                                                     
                                                    }),
              'curse touch': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','jinx'), 'advantage': ('temporary','jinx')})}),
              #nightmare powers
              'sheer horror': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('permanent','damage'), 'advantage': ('permanent','damage')})}),
              'thieving claw': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('permanent','damage'), 'advantage': ('permanent','loss of items')})}),
              'bestial claw': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('permanent','damage'), 'advantage': ('permanent','damage')})}),
              'mysterious claw': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('permanent','damage'), 'advantage': ('permanent','jinx')})}),
              'dirty claw': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('permanent','damage'), 'advantage': ('permanent','stress')})}),
              'deathly claw': ('n/a', 'n/a', [], {'melee' : ('melee', {'success' : ('permanent','damage'), 'advantage': ('permanent','fatigue')})}),
              #
              'club': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('temporary','fatigue')})}),
              'loaded gauntlet': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('temporary','fatigue')})}),
              'hand crossbow': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('temporary','damage'), 'advantage': ('temporary','fatigue')})}),
              'bow': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('temporary','damage'), 'advantage': ('tactical','disability')})}),
              'spear': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('tactical','imbalance')})}),
              'axe': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('tactical','damage')})}),
              'howl': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('tactical','stress'), 'advantage': ('temporary','stress')})}),
              'sting': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','imbalance'), 'advantage': ('permanent','fatigue')})}),
              'evil eye': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('temporary','jinx'), 'advantage': ('temporary','disability')})}),
              'dagger' : ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('tactical','loss of items')})}),
              'vicious bite': ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('permanent','fatigue')})}),
              'pistol': ('n/a', 'n/a', [], {'ranged' : ('shoot', {'success' : ('temporary','damage'), 'advantage': ('temporary','damage')})}),
              'flute' : ('n/a', 'n/a', [], {'melee': ('melee', {'success' : ('temporary','damage'), 'advantage': ('tactical','fatigue')})}),
             }

def new_feature(feature_type, name = None):
    shape, color, flags, interaction = feature_types.get(feature_type,
                        ('default', 'default', [],
                          new_action('feedback', {'msg': 'Default action.'})))
    if shape == 'default':
        debug.log("requested undefined feature type: " + str(feature_type))
    feature = Feature(shape = shape, color = color, flags = flags, interaction = new_action(interaction[0], interaction[1]))
    _name_piece(feature, feature_type, name)
    return feature

feature_types = {'n/a' : ('n/a', 'n/a', [], ('feedback', {'msg':('Implement this', 'n/a')})),
                 'wooden door' : ('closed door', 'wooden', ['closed', 'blocking', 'blocks sight'], ('door', {'open shape':'open door'})),
                 'ancient door' : ('closed door', 'dark wooden', ['closed', 'blocking', 'blocks sight'], ('door', {'open shape':'open door'})),
                 'sarcophagus': ('table', 'grave grey', ['blocking'], ('feedback', {'msg': ('This is an ominous stone sarcophagus.', 'grave grey')})),
                 'ancient desk' : ('table', 'dark wooden', ['blocking'], ('feedback', {'msg': ('This ancient wooden desk is covered in blasphemous inscriptions.', 'dark wooden')})),
                 'antique desk' : ('table', 'dark wooden', ['blocking'], ('feedback', {'msg': ('This antique wooden desk has piles of disorganised papers on it.', 'wooden')})),                 
                 'tombstone': ('tombstone', 'grave grey', ['blocking'], ('feedback', {'msg': ('The inscriptions on this tombstone eroded a long ago.', 'grave grey')})),
                 'withered tree' : ('tree', 'dark wooden', ['blocking'], ('feedback', {'msg': ('This tree is no more alive than any tombstone here.', 'dark wooden')})),
                 'gnarled tree' : ('tree', 'wooden', ['blocking', 'blocks sight'], ('feedback', {'msg': ('This tree is very gnarled.', 'wooden')})),
                 'shrubbery' : ('rubble', 'green', ['blocking'], ('feedback', {'msg': ('This shrubbery is low, but thick and thorny.', 'green')})),
                 'pile of trash' : ('rubble', 'dirty brown', ['blocking'], ('feedback', {'msg': ("That's an awful pile of trash. You don't want to step into that.", 'dirty brown')})),
                 'heap of trash' : ('statue', 'dirty brown', ['blocking', 'blocks sight'], ('feedback', {'msg': ("A lot of trash is piled up high here.", 'dirty brown')})),
                 'stall' : ('table', 'fashionable', ['blocking'], ('feedback', {'msg': ("There's a lot of garish shiny stuff presented on this stall. Criminals have awful taste.", 'fashionable')})),
                 'cart' : ('table', 'wooden', ['blocking'], ('feedback', {'msg': ('This cart is stuck in the dirt.', 'wooden')})),
                 'nightmarish tree': ('tree', 'light nightmare', ['blocking', 'blocks sight'], ('feedback', {'msg': ("That's the most horrific tree you could ever imagine.", 'light nightmare')})),
                 'horrific statue': ('statue', 'dark nightmare', ['blocking', 'blocks sight'], ('feedback', {'msg': ("You recoil in horror from the horrific statue.", 'dark nightmare')})),
                 'bed' : ('table', 'sandy', ['blocking'], ('feedback', {'msg': ("The Witch's bed. Quite ordinary, except for many bits of animal fur here and there.", 'sandy')})),
                 'table' : ('table', 'wooden', ['blocking'], ('feedback', {'msg': ("The Witch's table. There's an open cookbook here, as well as remnants of a previous meal.", 'wooden')})),
                 'window' : ('window', 'watery', ['blocking'], ('feedback', {'msg': ('You peek through a window.', 'watery')})),
                 'cauldron' : ('cauldron', 'grave grey', ['blocking'], ('feedback', {'msg': ("The Witch's cauldron. There's a nice smelling stew inside.", 'cursed yellow')})),
                 'filthy mattress' : ('table', 'cursed yellow', ['blocking'], ('feedback', {'msg': ("An extremely filthy straw bedding, no doubt the Piper's.", 'sandy')}))
                }

"""
'lab workstation' : ('terminal', 'white', ['blocking', 'blocks sight'], \
                                     ('mission', {'msg' : ('You sabotage lab equipment. No more stims from this workstation.','gold'), \
                                                  'done msg': ('You have already sabotaged this workstation.', 'gray'), \
                                                  'done color': 'gray',
                                                  'objective': 'lab sabotage'}))
"""

def new_agent(agent_type, name = None):
    shape, color, flags, traits, equipment = agent_types.get((agent_type),
                        ('default', 'default', ['blocking'],
                         {'health': 100}, [])    
                        ) 
    if shape == 'default':
        debug.log("requested undefined agent type: " + str(agent_type))
    agent = Agent(shape = shape, color = color, flags = flags, traits = traits)
    _name_piece(agent, agent_type, name)
    _add_necessary_traits(agent)
    for item_type in equipment:
        item = new_item(item_type)
        agent.receive(item)
        agent.equip(item)
    return agent

agent_types = {
               #denizens of the haunted graveyard
               'zombie' : ('zombie', 'grave grey', ['blocking'], {'ai': RelentlessMelee()},
                                 ['fist']),
               'skeleton' : ('skeleton', 'bone white', ['blocking'], {'ai': RelentlessMelee()},
                                 ['rusty blade']),
               'ghost' : ('ghost', 'deathly white', ['blocking'], {'ai': RelentlessMelee()},
                                 ['unearthly wail']),
               'poltergeist' : ('wisp', 'cursed yellow', ['blocking'], {'ai': StandardSoldier()},
                                 ['pebble', 'curse touch']),
               'dread bat' : ('bat', 'inky black', ['blocking'], {'ai': RelentlessMelee()},
                                 ['dread bite']),
               'the Necromancer' : ('person', 'inky black', ['blocking'], {'health': 15, 'ai': StandardSoldier()},
                                 ['bone wand']),
               #denizens of the forest
               'deerman' : ('beastman', 'sandy', ['blocking'], {'attitude': -3, 'ai': StandardSoldier()}, ['spear', 'bow']),
               'wolfman' : ('beastman', 'dusty gray', ['blocking'], {'ai': StandardSoldier()}, ['club', 'howl']),
               'boarman' : ('beastman', 'copper', ['blocking'], {'attitude': 3,'ai': RelentlessMelee()}, ['axe']),
               'wasp swarm' : ('swarm', 'cursed yellow', ['blocking'], {'ai': RelentlessMelee()}, ['sting']),
               'the Witch' : ('person', 'wooden', ['blocking'], {'health': 15, 'ai': StandardSoldier()},
                                 ['evil eye', 'club']),
               #denizens of the black market
               'fence' : ('criminal', 'fashionable', ['blocking'], {'attitude': -1, 'ai': StandardSoldier()}, ['dagger', 'hand crossbow']),
               'thug' : ('criminal', 'sandy', ['blocking'], {'attitude': 1, 'ai': StandardSoldier()}, ['loaded gauntlet', 'pebble']),
               'thief' : ('criminal', 'grave grey', ['blocking'], {'attitude': -2, 'ai': RelentlessMelee()}, ['dagger']),
               'guard dog' : ('dog', 'sandy', ['blocking'], {'attitude': 2, 'ai': RelentlessMelee()}, ['vicious bite']),
               'the Crime Lord' : ('criminal', 'copper', ['blocking'], {'health': 15, 'ai': StandardSoldier()},
                                 ['pistol', 'axe']),
               #denizens of the rat-infested landfill
               'rat hound' : ('rat', 'dirty brown', ['blocking'], {'attitude': -2, 'ai': RelentlessMelee()}, ['vicious bite']),
               'rabid ratling' : ('ratling', 'blood red', ['blocking'], {'attitude': 2, 'ai': RelentlessMelee()}, ['vicious bite']),
               'ratling warrior': ('ratling', 'copper', ['blocking'], {'attitude': 1, 'ai': StandardSoldier()}, ['rusty blade', 'hand crossbow']),
               'ratling scavenger': ('ratling', 'dirty brown', ['blocking'], {'attitude': -1, 'ai': StandardSoldier()}, ['rusty blade', 'hand crossbow']),
               'the Piper' : ('person', 'dirty brown', ['blocking'], {'health': 15, 'ai': StandardSoldier()},
                                 ['flute', 'hand crossbow']),
               #denizens of the nightmare world
               'the Fear After' : ('zombie', 'dark nightmare', ['blocking'], {'ai': RelentlessMelee()}, ['deathly claw']),
               'the Fear Below' : ('ratling', 'dark nightmare', ['blocking'], {'ai': RelentlessMelee()}, ['dirty claw']),
               'the Fear Outside' : ('beastman', 'dark nightmare', ['blocking'], {'ai': RelentlessMelee()}, ['bestial']),
               'the Fear Inside' : ('criminal', 'dark nightmare', ['blocking'], {'ai': RelentlessMelee()}, ['thieving claw']),
               'the Unknown Fear' : ('unknown', 'dark nightmare', ['blocking'], {'ai': RelentlessMelee()}, ['unknown claw']),
               'the Nightmare Tyrant' : ('tyrant', 'dark nightmare', ['blocking'], {'ai': NightmareTyrant()},
                                 ['sheer horror']),
              }

def new_terrain(terrain_type, name = None):
    shape, color, flags = terrain_types.get(terrain_type, ('default', 'default', []))
    if shape == 'default':
        debug.log("requested undefined terrain type: " + str(terrain_type))
    terrain = Terrain(shape, color, flags)
    _name_piece(terrain, terrain_type, name)
    return terrain

terrain_types = {'n/a' : ('n/a', 'n/a', []),
                 'graveyard fence' : ('wall', 'inky black', ['blocking']),
                 'graveyard soil' : ('floor', 'dirty brown', []),
                 'crypt wall' : ('wall', 'inky black', ['blocking', 'blocks sight']),
                 'crypt floor' : ('floor', 'dusty gray', []),
                 'memorial monument':('statue', 'grave grey', ['blocking', 'blocks sight']),
                 'toppled tombstone': ('floor', 'grave grey', []),
                 
                 'thorny thicket': ('wall', 'dark wooden', ['blocking', 'blocks sight']),
                 'grass' : ('floor', 'green', []),
                 'herb patch' : ('overgrown floor', 'green', []),
                 'small stream' : ('water', 'light watery', []),
                 'wooden floor' : ('floor','wooden',[]),
                 'wooden wall' : ('wall', 'wooden', ['blocking', 'blocks sight']),
                 'small, wooden fence': ('tombstone', 'dark wooden', ['blocking']),
                 
                 'slag heap of trash' : ('statue', 'dirty brown', ['blocking', 'blocks sight']),
                 'muddy puddle' : ('water', 'dirty brown', []),
                 'mound of trash' : ('statue', 'dirty brown', ['blocking', 'blocks sight']),
                 'dirt' : ('floor', 'dirty brown', []),
                 'cobblestone road': ('floor', 'dusty gray', []),
                 'stone wall': ('wall', 'dusty gray', ['blocking', 'blocks sight']),
                 'shelf full of illicit wares': ('wall', 'wooden', ['blocking', 'blocks sight']),
                 
                 'dreaming townsperson': ('person', 'light nightmare', ['blocking']),
                 'nightmarish ground' : ('floor', 'light nightmare', []),
                 'wicked weed' : ('overgrown floor', 'light nightmare', []),
                 'swirling horror' : ('rubble', 'dark nightmare', ['blocking', 'blocks sight']),
                 'golden floor' : ('floor', 'gold', []),
                 'golden wall' : ('wall', 'gold', ['blocking', 'blocks sight']),
                 'throne of nightmares' : ('throne', 'dark nightmare', ['blocking', 'blocks sight']),
                }


    
