# -*- coding: utf-8 -*-
'''
This code is part of NightmareTyrant; Copyright (C) 2013 Piotr 'ZasVid' Sikora; see NightmareTyrant.py for full notice.
'''

import libtcodpy as libtcod
import os
from config import *
from ui import UI
from sys import exc_info
from gameplay import GamePlay
from achievements import Achievements
from visuals import color

class GameSystem(object):
    '''
    This class handles setting the game up.
    '''

    def __init__(self):
        self.ui = UI()
        self.achievements = Achievements()

    def main_menu(self):
        img_left = libtcod.image_load(IMAGE_COSCOM)
        img_right = libtcod.image_load(IMAGE_RAVEN)
        while not libtcod.console_is_window_closed():
            #show the background image, at twice the regular console resolution
            self.ui.clear()
            #TODO: adjust for screen parameters
            libtcod.image_blit_2x(img_left, 0, 0, 2)
            libtcod.image_blit_2x(img_right, 0, 50, 2)
            
            #show the game's title, and some credits!
            libtcod.console_set_default_foreground(0, color('dark nightmare'))
            libtcod.console_set_alignment(0, libtcod.CENTER)
            libtcod.console_print(0, SCREEN_WIDTH/2, SCREEN_HEIGHT/2 - 4,  'NIGHTMARE TYRANT')
            libtcod.console_print(0, SCREEN_WIDTH/2, SCREEN_HEIGHT-2, 'by ZasVid')
            #show options and wait for the player's choice
            #choice = self.ui.menu('', [('Instructions','i'), ('Play a new game', 'p'), ('Continue last game','c'), ('Quit','q')], 24, indexing = UI.MENU_CUSTOM_INDEX)
            choice = self.ui.menu('', [('[C]redits', 'c'), ('Continue [l]ast game', 'l'), ('Play a [n]ew game', 'n'), ('[Q]uit','q')], \
                                  24, indexing = UI.MENU_NUMCOMBO_INDEX, default_text_color = 'light nightmare')
            if choice == 0: #credits? manual? something like that
                self.ui.msgbox("Credits : \n" \
                               "ZasVid - game design, programming\n" \
                               #"Irinka - writing \n" \
                               "\nThis work contains bits of open-source code (BSD license) from: \n" \
                               "* Joao F. Henriques (a.k.a. Jotaf)'s python + libtcod tutorial \n" \
                               "\nThanks to: \n" \
                               "* contributors to http://roguebasin.roguelikedevelopment.org\n" \
                               "* roguelike.phx.pl for years of fanning the flame of roguelike passion\n" \
                               "* and last but not least (quite the opposite, in fact), my greatest supporter, lovely Irineczka\n"
                               ,
                               width = SCREEN_WIDTH - 4,
                               default_text_color = 'light nightmare'
                               )#TODO: João
            elif choice == 2:  #new game
                self.gameplay = GamePlay(self.ui, self.achievements)
                self.gameplay.new()
                self.gameplay.play()
            elif choice == 1:  #load last game
                savefile = "savegame"
                if os.path.exists(savefile):
                    try:
                        self.gameplay = GamePlay(self.ui, self.achievements)
                        self.gameplay.load(savefile)
                    except:
                        self.ui.msgbox('\n Failed to load a save: \n'+str(exc_info()), 24)
                        continue
                    self.gameplay.play()
                else:
                    self.ui.msgbox('\n No saved game to load.\n',)
            elif choice == 3:  #quit
                break
